'' =================================================================================================
''
''   File....... laytonk_001.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               -- see below for terms of use
''   E-mail.....  
''   Started.... 
''   Updated.... 11 FEB 2015
''
'' =================================================================================================


con { speed & timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' ticks per second
  MS_001   = CLK_FREQ / 1_000                                    ' ticks per millisecond
  US_001   = CLK_FREQ / 1_000_000                                ' ticks per microsecond
  

con { io pins }

  RX1     = 31                                                   ' programming / debug port
  TX1     = 30                                                    
                                                                  
  SDA     = 29                                                   ' eeprom
  SCL     = 28                                                    
                                                                  
  OUT_EN  = 27                                                   ' outputs enable pin (low)     
                                                                  
  SIO     = 26                                                   ' TTL serial io
                                                                  
  RX2     = 23                                                   ' RS-485 port
  RX2_EN  = 22                                                    
  TX2_EN  = 21                                                    
  TX2     = 20                                                    
                                                                  
  R_LED   = 18                                                   ' bi-color LED
  G_LED   = 17                                                    
                                                                  
  OUT7    = 15                                                   ' outputs
  OUT6    = 14                                                    
  OUT5    = 13                                                    
  OUT4    = 12                                                    
  OUT3    = 11                                                    
  OUT2    = 10                                                    
  OUT1    =  9                                                    
  OUT0    =  8                                                    
                                                                  
  OPT_BR  =  7                                                   ' options swithes
  OPT_A1  =  6                                                    
  OPT_A0  =  5                                                    
  OPT_SM  =  4                                                    
                                                                  
  DMX_A8  =  3                                                   ' high-bit off DMX address
                                                                  
  DO_165  =  2                                                   ' control lines for 74x165
  CLK_165 =  1                                                    
  LD_165  =  0                                                    
                                                                  

con

   #0, OFF, RED, GRN, YEL                                       ' for r/g led

   #0, NOCON, RX, TX, RXTX                                      ' for rs-485

   #0, LSBFIRST, MSBFIRST                                       ' for 74x165 shift-register                                               


obj

' main { Spin }                                                 ' * main code loop

  io     : "jm_io"                                              '   basic io control
  time   : "jm_time"                                            '   for pause and timer
  prng   : "jm_prng"                                            '   pseudo-random number generator
  outs   : "jm_pwm8"                                            ' * pwm driver for outputs
  serial : "jm_fullduplexserial"                                ' * serial IO for EFX-TEK boards

' * = consumes Propeller cog when running
  
  
var


pub main  

  setup_io                                                       ' configure HC-8+ io pins
  setup_objects                                                  ' configure objects used

  repeat
    ap16_volume(%00, 50, 50)                                     ' set volume to 50%
    ap16_sfx(%00, 0, 0)                                          ' play SFX00 (ambient for prop) in loop
    fc4_reset(%00)                                               ' reset FC-4+
    fc4_level(%00, 0, 60)                                        ' ch0 to 24%
    fc4_level(%00, 1, 60)                                        ' ch1 to 24%

    repeat                                                       ' wait for trigger on IN0
      if (ttl_pin == 0)
        quit  

    fc4_fade(%00, 0, 60,  0, 2000)                               ' fade out ch0 in 2s
    fc4_fade(%00, 1, 60,  0, 2000)                               ' fade out ch1 in 2s
    fc4_fade(%00, 2,  0, 60, 2000)                               ' fade up ch3 in 2s
    fc4_fade(%00, 3,  0, 60, 2000)                               ' fade up ch4 in 2s

    outs.high(0)                                                 ' prop moves
    outs.high(2)                                                 ' strobe on
      
    ap16_volume(%00, 100, 100)   
    ap16_sfx(%00, 1, 0)                                          ' play SFX01

    outs.high(3)                                                 ' fog on
    outs.high(1)                                                 ' spitter on
    time.pause(5000)                                             ' wait 5s

    outs.low(3)                                                  ' fog off
    outs.low(1)                                                  ' spitter off
    time.pause(3000)                                             ' wait 3s

    fc4_fade(%00, 2, 60,  0, 1000)                               ' fade out in 1s 
    fc4_fade(%00, 3, 60,  0, 1000)

    outs.low(0)                                                  ' prop stop
    outs.low(2)                                                  ' strobe off

    ap16_reset(%00)                                              ' stop AP-16+
    time.pause(2000)

    fc4_fade(%00, 1,  0, 60, 2000)

    repeat                                                       ' wait for IN1
      if (ttl_pin == 1)
        quit                


pub setup_io

'' Configure for basic IO control

  outa[OUT7..OUT0] := %0000_0000                                ' clear outputs
  dira[OUT7..OUT0] := %1111_1111                                ' output mode                                  
  io.low(OUT_EN)                                                ' enable output driver

  setup_595                                                     ' setup x595 IO

  set_rgled(OFF)
  
  set_rs485(NOCON)                                              ' RS-485 not connected


pub setup_595

'' Setup IO pins for x595 shift registers
'' -- must be called from cog using the shift registers

  io.high(LD_165)                                               ' setup x165 io pins
  io.low(CLK_165)
  io.input(DO_165)
  
  
pub setup_objects

'' Setup objects used by the application

  time.start                                                     ' setup for pause
  prng.seed(-cnt, cnt ~> 4, $DEAD_BEEF, cnt << 4, cnt)           ' seed randomizer
  outs.start(8, OUT0)                                            ' start outputs driver
  serial.start(SIO, SIO, %1100, 38_400)                          ' serial for efx devices

  time.pause(50)                                                 ' let everything get started
  
  
con

  { --------------------------------- }
  {  S U P P O R T   R O U T I N E S  } 
  { --------------------------------- }


var

  long  lastscan                                                ' ttl/DMX input scan


pub ttl_pin | btns

'' Reads TTL header for 1-of-8 input
'' -- returns -1 (no input)
'' -- returns 0 to 7 for input on IN0 to IN7
''    * only one input allowed

  btns := ttl_inputs(true)                                      ' read ttl inputs

  if (bit_count(btns) == 1)                                     ' only one button pressed?
    return bit_pos(btns, 0)                                     '  yes, return button #
  else
    return -1                                                   '  none or bad input


pub ttl_inputs(rescan) 

'' Return TTL inputs status
'' -- updates global var "lastscan" if rescan is true

  if (rescan)
    lastscan := x165_in | (ina[DMX_A8] << 16)     ' read in0..in7, dmx addresss
   
  return (lastscan & $FF)                                       ' return TTL input bits


pub dmx_address(rescan) 

'' Return DMX address switch setting
'' -- updates global var "lastscan" if rescan is true

  if (rescan)
    lastscan := x165_in | (ina[DMX_A8] << 16)     ' read in0..in7, dmx addresss

  return (lastscan >> 8) & $01FF                                ' return DMX address bits
 

pub x165_in | tmp165

'' Input value from dual 74x165s on board
'' -- bit order is MSBFIRST      
                                                                                          
  outa[LD_165] := 0                                             ' blip Shift/Load line    
  outa[LD_165] := 1
         
  repeat 16                                                   ' read n bits         
    tmp165 := (tmp165 << 1) | ina[DO_165]                       ' get new bit             
    outa[CLK_165] := 1                                          ' blip clock              
    outa[CLK_165] := 0
   
  return tmp165 & $FFFF
  

pub option_inputs

'' Returns state of option switch bits (%0000..%1111, all off to all on)

  dira[OPT_BR..OPT_SM] := %0000                                 ' force to input mode

  return !ina[OPT_BR..OPT_SM] & $0F                             ' return state (1 = on)


pub set_rgled(state)

'' Sets R/G LED
'' -- direct control only

  case state                                                     ' set color to state
    OFF : outa[R_LED..G_LED] := %00
    RED : outa[R_LED..G_LED] := %10
    GRN : outa[R_LED..G_LED] := %01
    YEL : outa[R_LED..G_LED] := %00                              ' yellow requires active driver
   
      

pub set_rs485(state)

'' Sets RS-485 RX and TX enable inputs

  case state
    NOCON : outa[RX2_EN..TX2_EN] := %10
    RX    : outa[RX2_EN..TX2_EN] := %00 
    TX    : outa[RX2_EN..TX2_EN] := %11   
    RXTX  : outa[RX2_EN..TX2_EN] := %01

  dira[RX2_EN..TX2_EN] := %11


pub bit_count(value) | bc

'' Returns # of bits set (1) in value

  bc := 0                                                       ' clear count
  repeat 32                                                     ' test all bits
    bc += (value & %1)                                          ' add bit value to count
    value >>= 1                                                 ' next bit

  return bc


pub bit_pos(value, mode)

'' Returns position of 1st bit set (0..31)
'' -- mode 0 to scan from lsb, mode 1 to scan from msb 
'' -- -1 = no bits set

  if (value == 0)                                               ' if no bits                                   
    return -1                                                   '  return -1

  else
    if (mode == 0)                                              ' check from LSB
      value ><= 32                                              '  flip for >|
      return (32 - >|value)
    else
      return (>|value - 1)


pub bit_val(value, pos)

'' Returns bit (0..1) from pos (0..31) in value 

  if ((pos => 0) and (pos =< 31))
    return (value >> pos) & 1
  else
    return 0


con

  { ----------------------------- }
  {  F C - 4 +   R O U T I N E S  }
  { ----------------------------- }

  ' requires efx serial object to be started


dat

  FC4         byte    "!FC4", 0


pub fc4_reset(addr)

'' Sets all FC-4+ outputs at addr to off

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("X")


pub fc4_level(addr, ch, level)

'' Sets channel to level, 0 to 255

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("L")
  serial.tx(ch)
  serial.tx(0 #> level <# 255)


pub fc4_set_all(addr, level)

'' Sets all channels to level, 0 to 255

  fc4_level(addr, "A", level)                                    ' set all to level


pub fc4_fade(addr, ch, lvlstart, lvlstop, ms)

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("F")
  serial.tx("H")                                                 ' use high resolution fade
  serial.tx(ch)
  serial.tx(0 #> lvlstart <# 255)
  serial.tx(0 #> lvlstop <# 255)
  serial.tx(ms.byte[0])                                          ' send millis Little Endian
  serial.tx(ms.byte[1]) 


pub fc4_crossfade(addr, chA, chB, tenths)

'' Full cross-fade between channels chA and chB
'' -- channel chA starts at 100%
'' -- channel chB starts at   0%
'' -- timing specified in 0.1s (100ms) units    

  serial.str(@FC4)
  serial.tx(addr)
  serial.tx("F")
  serial.tx("X")
  serial.tx(chA)
  serial.tx(chB) 
  serial.tx(tenths)
    


con

  { ------------------------------- }
  {  A P - 1 6 +   R O U T I N E S  }
  { ------------------------------- }

  ' requires serial object to be started


dat

  AP16        byte    "!AP16", 0
    

pub ap16_status(addr)

'' Returns AP-16+ status byte

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("G")
  serial.rxflush

  return serial.rxtime(25)


pub ap16_playing(addr) | status

'' Returns playing status from AP-16+ at addr

  return ((ap16_status(addr) & %1000_0000) > 0)
  

pub ap16_sfx(addr, sfx, rpts)

'' Plays SFX file on AP-16+ at addr

  serial.str(@AP16)
  serial.tx(addr)
  serial.str(string("PS"))
  serial.tx(sfx)
  serial.tx(rpts)


pub ap16_aux(addr, aux, rpts)

'' Plays AUX file on AP-16+ at addr

  serial.str(@AP16)
  serial.tx(addr)
  serial.str(string("PA"))
  serial.tx(aux)
  serial.tx(rpts)


pub ap16_wav(addr, spntr, rpts)

'' Plays WAV file (name at spntr) on AP-16+ at addr

  serial.str(@AP16)
  serial.tx(addr)
  serial.str(string("PW"))
  serial.str(spntr)
  serial.tx(13)
  serial.tx(rpts)


pub ap16_volume(addr, left, right)

'' Sets AP-16+ volume levels

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("G")
  serial.tx(0 #> left <# 100)  
  serial.tx(0 #> right <# 100)  


pub ap16_reset(addr)

'' Stops AP-16+ at addr

  serial.str(@AP16)
  serial.tx(addr)
  serial.tx("X")


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      